# cache manager, sets cache, sync with remote cache


class cacheManager(object):

    def __init__(self):
        pass

    # Setters

    # Getters


class syncCache(object):
    """ Sync thread that constantly keeps local and remote cache in sync."""

    def __init__(self):
        pass

    def connect(self):
        """Connects to remote cache."""
        pass

    def sync(self):
        """ Performs the sync. """
        pass

    def cleanup_cache(self):
        """ Remove cache items if they are expired. """

        # check database tables for expired items, if expired delete record.

    def run(self):

        pass
