# for connecting to marketplace api

class marketplaceAPI(object):

    def __init__(self):
        pass

    def login(self):
        pass

    def query_plugins(self):
        """ Query available plugins.

        Gets a list of plugins for the marketplace account.

        """
        pass

    def query_local_plugins(self, filter=None):
        """ List local Plugins.
            Args:
                filter (str): One of the following [active, deactived, all]
            Returns:
                List of plugins, if filter is present returns only plugins meeting the filter.
        """
        pass

    def get_new_plugins(self):
        """Query Plugins and get list of new plugins to be installed."""

        return [for plugin in self.query_plugins() if plugin not in self.query_local_plugins(filter='all')]

    def download_plugin(self, uuid=None):
        """Download plugin."""

        pass
